# Import the modules
import os
import sys

import geofram as gf
import minvel as mv
import numpy as np
import thermod as tm

from ncm_database import database_directory
from netCDF4 import Dataset
from scipy import interpolate as interp
from scipy import special as sp


PathToDatabase = database_directory()


def GetJK(lat, lon, dim):
    fl = os.path.join(PathToDatabase, "NCM_SpatialGrid.nc")
    nc = Dataset(fl, "r")
    LAv = nc.variables["Latitude vector"][:]
    LOv = nc.variables["Longitude vector"][:]
    jg = nc.variables["Index j grid"][:]
    kg = nc.variables["Index k grid"][:]

    f1 = interp.interp2d(LOv, LAv, np.transpose(jg), kind="linear")
    f2 = interp.interp2d(LOv, LAv, np.transpose(kg), kind="linear")
    J = np.zeros(np.shape(lat), dtype=np.float64)
    K = np.zeros(np.shape(lat), dtype=np.float64)
    LocsOut = 0
    LocsEast = 0
    if dim == 1:
        nLocs = np.size(lat)
        for l in range(0, nLocs):
            J[l] = f1(lon[l], lat[l]) - 1
            K[l] = f2(lon[l], lat[l]) - 1
            if J[l] < 0 or J[l] > 3200 or K[l] < 0 or K[l] > 4900:
                LocsOut = 1
            if lon[l] > -100:
                LocsEast = 1
    else:
        nx, ny = np.shape(lon)
        for x in range(0, nx):
            for y in range(0, ny):
                J[x, y] = f1(lon[x, y], lat[x, y]) - 1
                K[x, y] = f2(lon[x, y], lat[x, y]) - 1
                if J[x, y] < 0 or J[x, y] > 3200 or K[x, y] < 0 or K[x, y] > 4900:
                    LocsOut = 1
                if lon[x, y] > -100:
                    LocsEast = 1

    if LocsOut == 1:
        print("One or more locations outside of the domain of the NCM.")
    if LocsEast == 1:
        print(
            "One or more locations are east of -100 where the geologic framework has not been finalized."
        )

    # There appears to be a difference in the projection algorithm between matlab and python. To maintain
    # consistency, I decide to interpolate the spatial index grid which was produced using matlab's
    # algorithms.
    # X1 = -2600000
    # Y1 = 0
    # X2 = 2300000
    # Y2 = 3200000
    # nx = 4900
    # ny = 3200

    # crs_4326 = CRS.from_epsg(4326)
    # crs = CRS.from_proj4("+proj=tmerc +lat_1=30 +lat_2=45 +lat_0=24 +lon_0=-95 +k_0=0.9996 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0")

    # projfwd = tr.from_crs(crs_4326, crs)
    # X,Y = projfwd.transform(lat,lon)
    # J = (ny - 1)*(Y - Y1)/(Y2 - Y1);
    # K = (nx - 1)*(X - X1)/(X2 - X1);

    return J, K


def loadPar(x1, x2, y1, y2):
    fl1 = os.path.join(PathToDatabase, "NCM_AuxData.nc")
    fl2 = os.path.join(PathToDatabase, "PetrologyDatabase.nc")
    nc = Dataset(fl1, "r")
    MNF1 = nc.variables["Calibration Slope"][:]
    MNF2 = nc.variables["Calibration Offset"][:]
    WTD = nc.variables["Water Table Depth"][x1:x2, y1:y2]

    nc = Dataset(fl2, "r")
    mi = nc.variables["Mineral Index"][:]
    vf = nc.variables["Volume Fraction"][:]

    MNF1[210:211] = 0
    MNF2[:, 210:211] = -5

    return MNF1, MNF2, WTD, mi, vf


def CalcGeo(lat, lon, Depth, igt):
    # Flags are as follows
    # Ignore Thermal Effect - 1(yes),0(no)

    ESP = 1.01e5

    nDepth = np.size(Depth)
    dim = np.size(np.shape(Depth))
    j, k = GetJK(lat, lon, dim)

    x1 = int(np.fmax(0, np.floor(np.fmin(4900, np.min(k))) - 5))
    x2 = int(np.fmin(4900, np.ceil(np.fmax(0, np.max(k))) + 5)) + 1
    y1 = int(np.fmax(0, np.floor(np.fmin(3200, np.min(j))) - 5))
    y2 = int(np.fmin(3200, np.ceil(np.fmax(0, np.max(j))) + 5)) + 1

    MNF1, MNF2, WTD, mi, vf = loadPar(x1, x2, y1, y2)

    xv = np.arange(x1, x2, 1)
    yv = np.arange(y1, y2, 1)

    f1 = interp.interp2d(xv, yv, np.transpose(WTD), kind="cubic")

    WaterTableDepth = np.zeros(np.shape(j), dtype=np.float64)
    nLocs = np.size(j)
    for l in range(0, nLocs):
        WaterTableDepth[l] = f1(k[l], j[l])

    Temp = np.zeros((nLocs, nDepth), dtype=np.float64)
    if igt == 1:
        KS = np.zeros((212), dtype=np.float64)
        US = np.zeros((212), dtype=np.float64)
        PS = np.zeros((212), dtype=np.float64)
        nm, nmi = np.shape(mi)
        P = 1e6 * np.ones((1), dtype=np.float64)
        T = 350 * np.ones((1), dtype=np.float64)
        for l in range(0, nmi):
            m = np.nonzero(mi[:, l] > 0)[0]
            if np.size(m) > 0:
                Ks, Us, E, lmb, v, Vp, Vs, Ps, Vpv, Vpr, Vsv, Vsr, a = mv.CalcMV(
                    mi[m, l], vf[m, l], T, P
                )
                KS[l] = Ks
                US[l] = Us
                PS[l] = Ps
            Ks, Us, E, lmb, v, Vp, Vs, Ps, Vpv, Vpr, Vsv, Vsr, a = mv.CalcMV(
                mi[0, 210], vf[0, 210], T, P
            )
            KS[210] = Ks
            US[210] = Us
            PS[210] = Ps
            Ks, Us, E, lmb, v, Vp, Vs, Ps, Vpv, Vpr, Vsv, Vsr, a = mv.CalcMV(
                mi[0, 211], vf[0, 211], T, P
            )
            KS[211] = Ks
            US[211] = Us
            PS[211] = Ps
        for l in range(0, nLocs):
            Temp[l, :] = 300 + 0.025 * Depth
    else:
        Temp = tm.CalcTemp(lat, lon, Depth) + 273.15

    Prof, Aux, rockLabel, ageLabel = gf.GeoProfile(lat, lon)

    Consts = np.zeros((11), dtype=np.float64)
    Consts[0] = 0  # Flag specifying whether to attenuate velocities, RPParms.Attenuate = 0
    Consts[1] = 0.5  # RPParms.pExp = 0.5

    GP = np.zeros((7, nLocs, nDepth), dtype=np.float32)
    for i in range(0, nLocs):
        Vs = np.zeros((nDepth), dtype=np.float64)
        Vp = np.zeros((nDepth), dtype=np.float64)
        Density = np.zeros((nDepth), dtype=np.float64)
        Porosity = np.zeros((nDepth), dtype=np.float64)
        DifferentialPressure = np.zeros((nDepth), dtype=np.float64)

        nPgOrg = np.size(np.nonzero(Prof[i, 0, :] > 0))
        NPG = nPgOrg
        ld = Aux[i, 0] - Prof[i, 2, np.arange(1, nPgOrg)]
        ld = np.concatenate([[0], ld])
        lt = np.diff(ld)

        LastVal = [nPgOrg - 1]
        K = np.nonzero(lt > 0.001)[0]
        ld = ld[K + 1]
        PG = np.int16(np.concatenate([Prof[i, 0, K], Prof[i, 0, LastVal]]))
        PA = np.int16(np.concatenate([Prof[i, 1, K], Prof[i, 1, LastVal]]))
        nPgOrg = np.size(PG)

        mnLD = min(1e-2, ld[0] / 2)
        mxLD = max(ld[nPgOrg - 2], 1e6)
        DepthVec = np.concatenate([[mnLD], ld, [mxLD]])
        DP = np.array([])
        VS = np.array([])
        VP = np.array([])
        RS = np.array([])
        PO = np.array([])
        PR = np.array([])
        Consts[2] = WaterTableDepth[i]  # RPParms.WaterTableDepth = wtd(i)
        for l in range(0, nPgOrg):
            Dep = np.logspace(
                np.log10(DepthVec[l] + min(1e-5, DepthVec[l] / 2)),
                np.log10(DepthVec[l + 1] - 1e-5),
                50,
            )
            Press = 2700 * 9.8 * Dep + ESP
            if igt == 1:
                Vecs = np.zeros((2, np.size(Dep)), dtype=np.float64)
                Consts[3] = KS[PG[l] - 1]
                Consts[4] = US[PG[l] - 1]
                Consts[5] = PS[PG[l] - 1]
                MinIndex = np.array([])
                VolFrac = np.array([])
            else:
                Vecs = np.zeros((3, np.size(Dep)), dtype=np.float64)
                m = np.nonzero(mi[:, PG[l] - 1] > 0)[0]
                MinIndex = mi[m, PG[l] - 1]
                VolFrac = vf[m, PG[l] - 1]
                if PG[l] > 210:
                    MinIndex = np.ones((1, 1))
                    VolFrac = np.ones((1, 1))
                    MinIndex[0] = mi[0, PG[l] - 1]
                    VolFrac[0] = vf[0, PG[l] - 1]
                f = interp.interp1d(Depth, Temp[i, :], kind="linear", fill_value="extrapolate")
                Vecs[2, :] = f(Dep)

            Vecs[0, :] = Dep
            Vecs[1, :] = Press

            Consts[6] = MNF1[PG[l] - 1]  # RPParms.p0 = MNF1(PG(l))
            Consts[7] = MNF2[PA[l] - 1, PG[l] - 1]  # RPParms.p0a = MNF2(PG(l),PA(l))
            if l == 0:
                Consts[8] = 0  # RPParms.fp0 = 0
                Consts[9] = ESP  # RPParms.rp0 = 1.01e5
                Consts[10] = 0  # RPParms.dp0 = 0
            else:
                Consts[8] = fluidPress[
                    np.size(Dep) - 1
                ]  # RPParms.fp0 = RP.FluidPressure(np.size(Dep))
                Consts[9] = rockPress[
                    np.size(Dep) - 1
                ]  # RPParms.rp0 = RP.RockPressure(np.size(Dep))
                Consts[10] = DP[np.size(DP) - 1]  # RPParms.dp0 = DP(np.size(DP))

            vs, vp, rs, por, fluidPress, rockPress, diffPress = RockProp(
                Vecs, Consts, MinIndex, VolFrac
            )
            DP = np.concatenate([DP, Dep])
            VS = np.concatenate([VS, vs])
            VP = np.concatenate([VP, vp])
            RS = np.concatenate([RS, rs])
            PO = np.concatenate([PO, por])
            PR = np.concatenate([PR, diffPress])

        f = interp.interp1d(DP, VS, kind="linear", fill_value="extrapolate")
        Vs = f(Depth)
        f = interp.interp1d(DP, VP, kind="linear", fill_value="extrapolate")
        Vp = f(Depth)
        f = interp.interp1d(DP, RS, kind="linear", fill_value="extrapolate")
        Density = f(Depth)
        f = interp.interp1d(DP, PO, kind="linear", fill_value="extrapolate")
        Porosity = f(Depth)
        f = interp.interp1d(DP, PR, kind="linear", fill_value="extrapolate")
        DifferentialPressure = f(Depth)

        GP[0, i, :] = Depth
        GP[1, i, :] = Vs
        GP[2, i, :] = Vp
        GP[3, i, :] = Density
        GP[4, i, :] = Porosity
        GP[5, i, :] = DifferentialPressure
        GP[6, i, :] = Temp[i, :] - 273.15

    return GP, WaterTableDepth


def RockProp(Vecs, Consts, MinIndex, VolFrac):
    #
    #    This program calculates the geophysical parameters of a porous mineral aggregate
    #    using one of 4 methods as indicated by the variable Parms.t, though t = 4 has been
    #    given the most attention.
    #
    #       OUTPUT (SI Units)
    #    RP   - structure of rock properties
    #                   Parms - Input parameter structure
    #           FluidPressure
    #            RockPressure
    #    DifferentialPressure
    #                Porosity
    #                       r - Mineral aggregate density
    #                       K - Mineral aggregate bulk modulus
    #                       G - Mineral aggregate shear modulus
    #                      rs - Bulk rock density
    #                      Ks - Bulk rock bulk modulus
    #                      Gs - Bulk rock shear modulus
    #                      Vp - P-wave velocity
    #                      Vs - S-wave velocity
    #                   att_p - Assummed P-wave attenuation to convert from anharmonic to seismic
    #                   att_s - Assummed S-wave attenuation to convert from anharmonic to seismic
    #
    #       INPUT (SI Units, unless specified otherwise)
    #    Parms - structure containing the following:
    #       Comp - composition structure--see MinVel
    #       p0   - zero pressure porosity
    #        NOTE: Be careful that if the porosity is too high, fluid pressure will increase faster than
    #        rock pressure. Recommend capping the porosity to prevent this.
    #       p0a  - porosity depth scaling in p0*exp(-p0a*dep)
    #       var  - matrix (3xn) containing the following vectors, each of length(n);
    #        Depth, Pressure, Temperature (m,Pa,K)
    #        If matrix is 1xn (Depth only), Pressure and Temperature are calculated
    #            in the following way:
    #            Pressure = 1.01e5 + Depth*9.8*2700;
    #            Temp = 300+0.022*Depth;
    #        NOTE: Depth vector is the midpoint depth
    #          t    - Type of relation
    #               1 - Toksoz et al, 1976
    #        2 - Budiansky and O'Connel
    #        3 - Myung Lee (2003)
    #        4 - Myung Lee (2010), currently used for NCM
    #
    #          The following parameters are optional. Default values are 0 if not specified.
    #       WaterTableDepth - Water table depth (m). If specifying a profile beginning at some depth where
    #           fp0 includes the effect of the water table, set wtd = 0.
    #       VadoseZoneThickness - Thickness of partially saturated region above the water table.
    #       rp0  - Rock pressure at the top of the column, should be earth surface pressure
    #           if column begins at the surface.
    #       fp0  - fluid pressure at the top of the column, should be earth surface pressure
    #           if column begins at the surface.
    #       a    - aspect ratio of inclusions, if t == 1 or 2
    #
    #          The following parameters are optional but, if specified, must be specified together
    #       att_p - P-wave attenuation
    #       att_s - S-wave attenuation
    #       qf    - Frequency dependence of attenuation
    #
    #       The following Parms fields may be specified in order to avoid call to MinVel
    #       K       - bulk modulus
    #       G       - shear modulus
    #       Density - density
    #

    # For Lee's method, another useful article to cite is
    # Luo, X., Were, P., Liu, J. et al. Environ Earth Sci (2015) 73: 7019. doi:10.1007/s12665-015-4219-8
    # For comparisons with Prasad's data, a composition of
    # Comp(8,[1 5 66]) = [0.4 0.2 0.4]; % works best
    ncol, nDepth = np.shape(Vecs)
    Vs = np.zeros((nDepth), dtype=np.float64)
    Vp = np.zeros((nDepth), dtype=np.float64)
    rs = np.zeros((nDepth), dtype=np.float64)
    p = np.zeros((nDepth), dtype=np.float64)
    RockPRessure = np.zeros((nDepth), dtype=np.float64)
    FluidPressure = np.zeros((nDepth), dtype=np.float64)
    DifferentialPressure = np.zeros((nDepth), dtype=np.float64)

    # maxPor = 0.8;
    Gravity = 9.8
    ESP = 1.01e5  # pressure at Earth's surface
    minDiffP = ESP

    # Dry
    # up = 0;
    # Kp = 0;
    # rp = 0;

    # Water as the assumed inclusion
    rFluid = 1e3
    rAir = 1.225
    KFluid = 2.29e9
    KAir = 1.42e5
    up = 0

    Kp = KFluid * np.ones((nDepth), dtype=np.float64)
    rp = rFluid * np.ones((nDepth), dtype=np.float64)

    MaxSuction = -1000e3
    # Pascals
    MinSaturation = 0
    minPorSlope = 0.1

    pExp = Consts[1]
    p0 = Consts[6]
    p0a = Consts[7]
    WaterTableDepth = Consts[2]
    fp0 = Consts[8]  # Initial fluid pressure
    rp0 = Consts[9]  # Initial rock pressure
    dp0 = Consts[10]  # Depth at which initial pressures are valid
    CapillaryFringeThickness = min(0.5, 0.5 * WaterTableDepth)
    VadoseZoneThickness = WaterTableDepth - CapillaryFringeThickness

    # Considered looping this since the pressure would be dependent on the density change,
    # but the change in mineral density and moduli is extremely small.
    Depth = Vecs[0, :]
    Pressure = Vecs[1, :]
    if ncol == 3:
        Temp = Vecs[2, :]
        K, u, E, lmb, v, mvp, mvs, r, Vpv, Vpr, Vsv, Vsr, a = mv.CalcMV(
            MinIndex, VolFrac, Temp, Pressure
        )
    else:
        K = Consts[3]
        u = Consts[4]
        r = Consts[5]

    maxPor = np.amin(r) / (rFluid + np.amin(r))

    ggd = Gravity * np.diff(np.concatenate([[dp0], Depth]))
    wden_rock = r * (1 - 0.3)
    wden_fluid = rFluid * 0.3
    I1 = WaterTableDepth
    I2 = I1 - CapillaryFringeThickness
    I3 = I2 - VadoseZoneThickness
    wti = np.nonzero(Depth <= I1)[0]  # Above the Water Table
    wti2 = np.nonzero((Depth <= I1) & (Depth >= I2))[0]  # In the Capillary fringe
    wti3 = np.nonzero((Depth < I2) & (Depth >= I3))[0]  # In the Vadose Zone
    wti4 = np.nonzero(Depth < I3)[0]  # Above the Vadose Zone
    nwti = np.size(wti)
    nwti2 = np.size(wti2)
    nwti3 = np.size(wti3)
    nwti4 = np.size(wti4)
    # Sand
    # a = 0.3;
    # n = 3;
    # Silt
    # a = 0.05;
    # n = 2.5;
    # Clay
    # a = 0.01;
    # n = 1.8;
    if nwti > 0:  # Material above the water table
        a = 0.3  # Sand from Lu et al. 2010
        n = 3  # Sand from Lu et al. 2010
        WaterTablePressure = Gravity * WaterTableDepth * wden_fluid
        FluidPressure = np.cumsum(ggd * wden_fluid) - WaterTablePressure
        # QUESTION: How does saturation change with depth in the vadose zone?
        # Related answer: John McCartney presented work on the partially saturated zone in a USGS seminar.
        #    In the presentation, he showed water retention curves and provided me a paper on the subject.
        if nwti3 > 0:  # The vadose zone
            x = np.pi * ((Depth[wti3] - I3) / (I2 - I3)) - np.pi / 2
            Se = (np.sin(x) + 1) / 2
            Se = Se**2
            dn = 1 - Se
            # I multiplied Sc by saturation because I couldn't match borehole velocities
            # otherwise. I think I could accomplish this by modifying the saturation profile.
            Sc = np.zeros(np.size(Se), dtype=np.float64)
            j = np.nonzero(Se > 0)[0]
            Sc[j] = -(Se[j] / a) * (Se[j] ** (n / (1 - n)) - 1) ** (1 / n)
            # Sc = Sc.*Se;
            rp[wti3] = dn * (rAir - rFluid) + rFluid
            k = np.nonzero(rp < rAir)[0]
            rp[k] = rAir
            Kp[wti3] = dn * (KAir - KFluid) + KFluid
            k = np.nonzero(Kp < KAir)[0]
            Kp[k] = KAir
            FluidPressure[wti3] = Sc * 1e3
        else:
            dn = 0

        if nwti4 > 0:  # Above the vadose zone
            rp[wti4] = rAir
            Kp[wti4] = KAir
            FluidPressure[wti4] = fp0
        # There are two pressures here that add to the rock pressure, the weight of the fluid and the suction.
        # According to McCartney's presentation, suction varies linearly through the vadose zone, reaching a
        # maximum at the top of the phreatic zone (?) and decreasing to zero at the top.
        # See Lu et al, 2010 for suction as a function of saturation for different deposits.
    else:
        dn = 0
        FluidPressure = fp0 + np.cumsum(ggd * wden_fluid)

    RockPressure = rp0 + np.cumsum(ggd * wden_rock)
    # Differential pressure is referred to as effective pressure in Lu et al. 2010.
    DifferentialPressure = RockPressure - FluidPressure
    k = np.nonzero(DifferentialPressure < minDiffP)[0]
    DifferentialPressure[k] = minDiffP
    TopVZ = WaterTableDepth - CapillaryFringeThickness - VadoseZoneThickness
    BottomVZ = WaterTableDepth - CapillaryFringeThickness

    p0aMax = np.log(-np.log(maxPor))
    for j in range(0, 4):
        EfP = DifferentialPressure / 101000
        maxPorFunc = np.exp(minPorSlope * np.log(EfP) ** pExp + p0aMax)
        p = np.exp(p0 * np.log(EfP) ** pExp + p0a)
        p = (p**10 + maxPorFunc**10) ** (1 / 10)
        p = np.exp(-p)

        wden_rock = r * (1 - p)
        wden_fluid = rp * p
        if nwti > 0:
            WaterTablePressure = np.sum(ggd[wti] * wden_fluid[wti])
            FluidPressure = np.cumsum(ggd * wden_fluid) - WaterTablePressure
            FluidPressure[wti4] = fp0
            if nwti3 > 0:
                SC = Sc * 1e3 + FluidPressure[wti3[nwti3 - 1]] * (Depth[wti3] - TopVZ) / (
                    BottomVZ - TopVZ
                )
                FluidPressure[wti3] = SC

        else:
            FluidPressure = fp0 + np.cumsum(ggd * wden_fluid)

        RockPressure = rp0 + np.cumsum(ggd * wden_rock)
        DifferentialPressure = RockPressure - FluidPressure
        k = np.nonzero(DifferentialPressure < minDiffP)[0]
        DifferentialPressure[k] = minDiffP

    rs = wden_rock + rp * p

    # Original coeffiecients for Lee when porosity in Biot coeffients are not normalized by maxPor
    # A1 = 38;
    # A2 = 100;
    # When using p/maxPor in Biot coefficients, usign grid search to match S-velocity curves when
    # A1 = 38 and A2 = 100, p0 = 0.3941, p0a = 0, and pExp = 0.5;
    # Coefficients for matching P-vel are A1 = 25.15 and A2 = 70.50 (incr of 0.05);
    # A1 = 25.05
    # A2 = 71.20
    A1 = 11.025288436095856
    A2 = 30.825849167011931
    A2exp = -0.4

    M1 = 2
    M2 = 9.723002325376470
    M2exp = -0.6

    TransExp = 10
    DP = DifferentialPressure * 1e-6  # Differential Pressure in MPa

    a1 = A1
    a2 = A2 * DP**A2exp
    # QUESTION: What is the correct value of a3?
    # a3 = 2
    a12 = 1 / (((1 / a1) ** TransExp + (1 / a2) ** TransExp) ** (1 / TransExp))
    # A = (a12**TransExp + a3**TransExp)**(1/TransExp)
    A = a12

    Bp = (p / maxPor) * (1 + A) / (1 + A * (p / maxPor))

    # m2 and m3 original functions from Lee (2010)
    m1 = A1 - 0 * DP
    m2 = M2 * (DP) ** M2exp
    m3 = M1 - 0.0 * DP

    # m12 = 1/(((1/m1)**TransExp + (1/m2)**TransExp)**(1/TransExp))
    m12 = m2
    M = (m12**TransExp + m3**TransExp) ** (1 / TransExp)
    g = (1 + M * A) / (1 + A)

    # Lee contends that above the water table, g = 1
    # This assumption produces a large jump (> 200 m/s) in velocity at the water table
    # that is not seen in the velocity profiles.
    # g(wti) = 1;

    Bs = (p / maxPor) * (1 + g * A) / (1 + g * A * (p / maxPor))

    Bp[wti4] = Bs[wti4]
    Bp[wti3] = Bp[wti3] - dn * (Bp[wti3] - Bs[wti3])

    Kav = 1 / ((Bp - (p / maxPor)) / K + (p / maxPor) / Kp)
    Ks = K * (1 - Bp) + Kav * Bp**2
    us = u * (1 - Bs)

    Vp = np.sqrt((Ks + 4 * us / 3) / rs)
    Vs = np.sqrt(us / rs)

    # QUESTION: What attenuation functions should be used for p- and s-waves?
    # if Consts[0] == 1: # Attenuate velocities
    # if isfield(Parms,'AagaardModel') == 1:
    # Aagaard et al. functionality from the bay area
    # Qs = -16 + 104.13*Vs/1e3 - 25.225*(Vs/1e3)**2 + 8.2184*(Vs/1e3)**3
    # Qs = max(5,Qs)
    # Qp = 2*Qs
    # att_p = 1/Qp
    # att_s = 1/Qs
    # qf = 0

    # Parms.att_p = att_p
    # Parms.att_s = att_s
    # Parms.qf = qf
    # else:
    # att_p = Parms.att_p
    # att_s = Parms.att_s
    # qf = Parms.qf
    # Vp = Vp.*(1 - att_p*0.5*np.cos(pi*qf/2))
    # Vs = Vs.*(1 - att_s*0.5*np.cos(pi*qf/2))

    return Vs, Vp, rs, p, FluidPressure, RockPressure, DifferentialPressure
