# Import the modules
import argparse
import os
import sys
import time

import numpy as np

from ncm_database import check_geo_phys_database

import geophys.geo_phys as gp


def list_str(values):
    return values.split(",")


ProgDescr = """
GeoPhys -- Program to calculate geophysical profiles within the National Crustal Model. Profiles
can be returned from roughly -126 degrees to -63 degrees longitude and 22 to 53 degrees latitude.
If the requested location is outside of the bounds of the NCM, a message to that effect will be
returned.

Written by Oliver Boyd, v1. 19/10/12
"""

ProgEpilog = """
OUTPUT: At the locations requested.
  Vs - Shear-wave velocity (m/s)
  Vp - P-wave velocity (m/s)
  Density - (kg/m3)
  Porosity - Fraction from zero to one
  Differential Pressure - (Pa)
  Temperature - (celcius)
  Water Table Depth - (m)

Examples:
  Geophysical profile at a latitude and longitude of 36, -118 from the surface to 20 km depth in
  increments of 100 m.
  > poetry run geophys -l 36,-118 -d 0,1e2,20e3

  Geophysical map at a depth of 10 km on a 0.02 degree grid from 30 to 40 degrees latitude
  and -125 to -115 degrees longitude
  > poetry run geophys -l 30,0.02,35,-125,0.02,-120 -d 1e3
"""

LocHelp = """Location(s) of profile(s). Multiple individual locations are specified with
the first type. A map is specified with the second, and only one set can be specified."""

DepHelp = "Depth vector specified for profiles or the depth at which a map is produced"

FileHelp = """
File name containing profile information with the format:
  MinDepth,DepthIncrement,MaxDepth
  Lat1,Lon1
  Lat2,Lon2
  LatN,LonN
"""

OutHelp = "Directory in which to write results"

parser = argparse.ArgumentParser(
    formatter_class=argparse.RawTextHelpFormatter, description=ProgDescr, epilog=ProgEpilog
)
parser.add_argument(
    "-l",
    metavar="Lat,Lon or LatMin,LatInc,LatMax,LonMin,LonInc,LonMax",
    type=list_str,
    nargs="+",
    dest="Location",
    help=LocHelp,
)
parser.add_argument(
    "-d",
    metavar="DepthMin,DepthInc,DepthMax or Depth",
    nargs=1,
    type=list_str,
    dest="DepthVector",
    help=DepHelp,
)
parser.add_argument("-f", metavar="Filename", type=str, dest="InputFile", help=FileHelp)
parser.add_argument(
    "-od", metavar="Output directory", type=str, dest="outDir", default=".", help=OutHelp
)


def main():
    check_geo_phys_database()
    t1 = time.time()
    args = parser.parse_args()

    outDir = args.outDir
    if args.Location == None and args.DepthVector == None and args.InputFile == None:
        print("There are no arguments. See program help")
        exit()
    if (args.Location == None or args.DepthVector == None) and args.InputFile == None:
        print("Must specify location and depth vector or input file. See program help")
        exit()
    if args.Location != None:
        if len(args.Location[0]) != 2 and len(args.Location[0]) != 6:
            print("Location not specified correctly. See program help")
            exit()
        if len(args.Location[0]) == 2:
            nLocs = len(args.Location)
            for j in range(0, nLocs):
                if len(args.Location[j]) != 2:
                    print("Location must be lat,lon pairs. See program help")
                    exit()
    if args.DepthVector != None:
        if len(args.DepthVector[0]) != 1 and len(args.DepthVector[0]) != 3:
            print("Depth vector not specified correctly. See program help")
            exit()

    if args.Location != None:
        if len(args.Location[0]) == 2:
            profile = 1
            lat = np.zeros((nLocs), dtype=np.float64)
            lon = np.zeros((nLocs), dtype=np.float64)
            for j in range(0, nLocs):
                lat[j] = args.Location[j][0]
                lon[j] = args.Location[j][1]
        else:
            profile = 0
            latm = np.zeros((1), dtype=np.float64)
            lati = np.zeros((1), dtype=np.float64)
            latM = np.zeros((1), dtype=np.float64)
            lonm = np.zeros((1), dtype=np.float64)
            loni = np.zeros((1), dtype=np.float64)
            lonM = np.zeros((1), dtype=np.float64)
            latm[0] = args.Location[0][0]
            lati[0] = args.Location[0][1]
            latM[0] = args.Location[0][2]
            lonm[0] = args.Location[0][3]
            loni[0] = args.Location[0][4]
            lonM[0] = args.Location[0][5]
            nLon = int(np.round((lonM - lonm) / loni) + 1)
            nLat = int(np.round((latM - latm) / lati) + 1)
            [lon, lat] = np.meshgrid(
                np.linspace(lonm, lonM, num=nLon), np.linspace(latm, latM, num=nLat)
            )
            nLocs = np.shape(lon)

    if args.DepthVector != None:
        if len(args.DepthVector[0]) == 3:
            depmin = np.zeros((1), dtype=np.float64)
            depinc = np.zeros((1), dtype=np.float64)
            depmax = np.zeros((1), dtype=np.float64)
            depmin[0] = args.DepthVector[0][0]
            depinc[0] = args.DepthVector[0][1]
            depmax[0] = args.DepthVector[0][2]
        else:
            depval = np.zeros((1), dtype=np.float64)
            depval[0] = args.DepthVector[0][0]

    elif args.InputFile != None:
        print("Reading {0:s}".format(args.InputFile))
        f = open(args.InputFile, "r")
        if f.mode == "r":
            nLocs = 0
            ln = 0
            for line in f:
                line = line.strip()
                columns = line.split(",")
                if ln == 0:
                    if np.size(columns) == 3:
                        depmin = np.zeros((1), dtype=np.float64)
                        depinc = np.zeros((1), dtype=np.float64)
                        depmax = np.zeros((1), dtype=np.float64)
                        depmin[0] = columns[0]
                        depinc[0] = columns[1]
                        depmax[0] = columns[2]
                        profile = 1
                    else:
                        depval[0] = columns[0]
                        profile = 0
                else:
                    nLocs = nLocs + 1
                ln = ln + 1
        f.close()
        if profile == 1:
            lat = np.zeros((nLocs), dtype=np.float64)
            lon = np.zeros((nLocs), dtype=np.float64)
        else:
            latm = np.zeros((1), dtype=np.float64)
            lati = np.zeros((1), dtype=np.float64)
            latM = np.zeros((1), dtype=np.float64)
            lonm = np.zeros((1), dtype=np.float64)
            loni = np.zeros((1), dtype=np.float64)
            lonM = np.zeros((1), dtype=np.float64)
        f = open(args.InputFile, "r")
        if f.mode == "r":
            ln = 0
            jT = 0
            for line in f:
                line = line.strip()
                columns = line.split(",")
                if ln > 0:
                    if profile == 1:
                        lat[jT] = columns[0]
                        lon[jT] = columns[1]
                        jT = jT + 1
                    else:
                        latm[0] = entries[0]
                        lati[0] = entries[1]
                        latM[0] = entries[2]
                        lonm[0] = entries[3]
                        loni[0] = entries[4]
                        lonM[0] = entries[5]
                        nLon = int(np.round((lonM - lonm) / loni) + 1)
                        nLat = int(np.round((latM - latm) / lati) + 1)
                        [lon, lat] = np.meshgrid(
                            np.linspace(lonm, lonM, num=nLon), np.linspace(latm, latM, num=nLat)
                        )
                ln = ln + 1
        f.close()

    if os.path.exists(outDir) == 0:
        createDir = input("Output directory does not exist. Would you like to create it (y)/n?")
        if createDir == "n":
            sys.exit()
        else:
            try:
                os.mkdir(outDir)
                print(outDir, " created")
            except:
                print("Unable to create ", outDir, " Exiting")
                sys.exit()

    if profile == 0:
        Depth = depval * np.ones(np.shape(lon))
    else:
        nDepth = int(np.round((depmax - depmin) / depinc) + 1)
        Depth = np.linspace(depmin, depmax, num=nDepth)
        Depth = np.reshape(Depth, nDepth)

    GP, WaterTableDepth = gp.CalcGeo(lat, lon, Depth, 0)

    print("Saving geophysical information in %s/Depth.npy and GeoP.npy" % (outDir))
    np.save("%s/Depth.npy" % (outDir), Depth)
    np.save("%s/GeoP.npy" % (outDir), GP)

    for k in range(0, np.size(lat)):
        print("Profile #", k + 1)
        print("WaterTableDepth %10.2f meters" % (WaterTableDepth[k]))
        print("Depth      S-Velocity P-Velocity    Density   Porosity Diff Press Temperature")
        for j in range(0, np.size(Depth)):
            print(
                "%10.2f %10.2f %10.2f %10.2f %10.3f %8.4e %10.2f"
                % (
                    GP[0, k, j],
                    GP[1, k, j],
                    GP[2, k, j],
                    GP[3, k, j],
                    GP[4, k, j],
                    GP[5, k, j],
                    GP[6, k, j],
                )
            )

    elapsed_time = time.time() - t1
    print("Elapsed time: %10.2f seconds" % (elapsed_time))
    sys.exit()
