function GP = GeoPhys(Parms)
%function GP = GeoPhys(Parms)
%
% Return profile from the NCM with geophysical information
%
% Profiles can be returned from roughly -126 degrees to -63 degrees longitude and
% 22 to 53 degrees latitude. If the requested location is outside of the bounds of
% the NCM, a message to that effect will be returned.
%
% The following globals must be defined
% global MNF1 MNF2
%
% INPUT: Parms structure containing
%       j,k - vector of locations on NCM grid.
%               OR
%       lat,lon - latitude and longitude of points of interest
%
%       Depth - depth vector in meters (optional). If not specified, a depth vector of 
%	    0.01 to twice the maximum layer depth.
%
%	IgnoreThermalEffect - will ignore the thermal effect if this field is present
%

global MNF1 MNF2 WTD Rock

if isfield(Parms,'lon')
	[j,k] = GetJK(Parms.lat,Parms.lon);
	if length(find(isnan(j) == 1)) > 0 | length(find(isnan(k) == 1)) > 0
		fprintf('One or more locations outside of National Crustal Model domain.\n');
	end
	if length(find(Parms.lon > -100)) > 0
		fprintf('One or more locations are east of -100 where the geologic framework has not been finalized.\n');
	end
else
	j = Parms.j;
	k = Parms.k;
	if length(find(j < 1)) > 0 | length(find(j > 3201)) > 0 | length(find(k < 1)) > 0 | length(find(k > 4901)) > 0
		l = find(j < 1); j(l) = 1;
		l = find(j > 3201); j(l) = 3201;
		l = find(k < 1); k(l) = 1;
		l = find(k > 4901); k(l) = 4901;
		fprintf('One or more locations outside of National Crustal Model domain.\n');
	end
end
if isfield(Parms,'Geology')
	GeologySpecified = logical(1);
else
	GeologySpecified = logical(0);
end
if isfield(Parms,'p0a')
	p0aSpecified = logical(1);
else
	p0aSpecified = logical(0);
end
if isfield(Parms,'wtd')
	if length(j) > 1 & length(Parms.wtd) == 1
		wtd = ones(size(j))*Parms.wtd;
	else
		wtd = Parms.wtd;
	end
else
	if round(j) == j & round(k) == k
		I = sub2ind([3201 4901],j,k);
		wtd = double(WTD(I));
	else
		wtd = interp2(1:4901,1:3201,WTD,k,j,'cubic');
	end
end

if isfield(Parms,'Depth') == 0
	Parms.Depth = logspace(-2,log10(5e4),100);
end

if isfield(Parms,'IgnoreThermalEffect')
	KS = zeros(1,212);
	US = zeros(1,212);
	PS = zeros(1,212);
	for l = 1:length(Rock)
		if length(Rock(l).MinIndex) > 0
			Comp.Min = Rock(l).MinIndex;
			Comp.Fr = Rock(l).VolFrac;
			MV = MinVel(Comp,1e6,350);
			KS(l) = MV.K;
			US(l) = MV.G;
			PS(l) = MV.p;
		end
	end
	igt = 1;
else
	igt = 0;
end

RPParms.t = 4;
RPParms.Attenuate = 0;
RPParms.pExp = 0.5;
np = length(j);
t0 = tic;

nDepth = length(Parms.Depth);
Vs = zeros(np,nDepth);
Vp = zeros(np,nDepth);
Density = zeros(np,nDepth);
Porosity = zeros(np,nDepth);
DifferentialPressure = zeros(np,nDepth);
Temperature = zeros(np,nDepth);
WaterTableDepth = zeros(1,np);
for i = 1:np
	if np > 5000 & mod(i,5e3) == 0
		tt = toc(t0);
		fprintf('%8d %4d %4d %10.2f\n',i,j(i),k(i),tt)
		t0 = tic;
	end
	GParms.j = j(i);
	GParms.k = k(i);
	Prof = GeoFram(GParms);
	if GeologySpecified
		Prof.Geology = Parms.Geology(i,:);
		Prof.Age = Parms.Age(i,:);
		Prof.LayerElevation = Prof.Elev - Parms.Dep(i,:);
	end

	if igt == 0
		TParms.j = j(i);
		TParms.k = k(i);
		if isfield(Parms,'Depth')
			TParms.Depth = Parms.Depth;
		else
			TParms.Depth = logspace(-2,log10(2*max(Prof.Elev - Prof.LayerElevation)),200);
		end
		TParms.Temp = TherMod(TParms)+273.15;
	end

	nPgOrg = length(find(Prof.Geology > 0));
	NPG = nPgOrg;
	ld = double(Prof.Elev - Prof.LayerElevation(1:nPgOrg-1));
	lt = diff([0 ld]);
	%K = find(ld > 0.01);
	%ld = ld(K);
	%PG = [Prof.Geology(K) Prof.Geology(nPgOrg)];
	%PA = [Prof.Age(K) Prof.Age(nPgOrg)];

	%nPgOrg = length(PG);
	%K = find(lt == 0);
	%rmL = zeros(1,length(ld));
	%rmL(K+1) = 1;
	%K = find(rmL == 0);
	K = find(lt > 0.001);
	ld = ld(K);
	PG = [Prof.Geology(K) Prof.Geology(nPgOrg)];
	PA = [Prof.Age(K) Prof.Age(nPgOrg)];
	nPgOrg = length(PG);

	Depth = [min(1e-2,ld(1)/2) ld max(ld(nPgOrg-1),1e6)];
	DP = [];
	VS = [];
	VP = [];
	RS = [];
	PO = [];
	PR = [];
	RPParms.WaterTableDepth = wtd(i);

	for l = 1:nPgOrg
		Dep = logspace(log10(Depth(l)+min(1e-5,Depth(l)/2)),log10(Depth(l+1)-1e-5),50);
		Press = 2700*9.8*Dep;
		if igt == 1
 			RPParms.K = KS(PG(l));
			RPParms.G = US(PG(l));
			RPParms.Density = PS(PG(l));
			Temp = 273+0.025*Dep;
		else
			RPParms.Comp.Min = Rock(PG(l)).MinIndex;
			RPParms.Comp.Fr = Rock(PG(l)).VolFrac;
			Temp = interp1(TParms.Depth,TParms.Temp,Dep,'linear','extrap');
		end
		RPParms.var = [Dep; Press; Temp];
		RPParms.p0 = double(MNF1(PG(l)));
		if p0aSpecified
			RPParms.p0a = Parms.p0a(i,l);
		else
			RPParms.p0a = double(MNF2(PG(l),PA(l)));
		end
		if l == 1
			RPParms.fp0 = 0;
			RPParms.rp0 = 1.01e5;
			RPParms.dp0 = 0;
		else
			RPParms.fp0 = RP.FluidPressure(length(Dep));
			RPParms.rp0 = RP.RockPressure(length(Dep));
			RPParms.dp0 = DP(length(DP));
		end
		RP = RockProp(RPParms);
		DP = [DP Dep];
		VS = [VS RP.Vs];
		VP = [VP RP.Vp];
		RS = [RS RP.rs];
		PO = [PO RP.Porosity];
		PR = [PR RP.DifferentialPressure];
	end
	if isfield(Parms,'Depth')
		Vs(i,:) = interp1(DP,VS,Parms.Depth,'linear','extrap');
		Vp(i,:) = interp1(DP,VP,Parms.Depth,'linear','extrap');
		Density(i,:) = interp1(DP,RS,Parms.Depth,'linear','extrap');
		Porosity(i,:) = interp1(DP,PO,Parms.Depth,'linear','extrap');
		DifferentialPressure(i,:) = interp1(DP,PR,Parms.Depth,'linear','extrap');
		if igt == 0
			Temperature(i,:) = TParms.Temp;
		end
		WaterTableDepth(i) = wtd(i);
	else
		GP(i).Depth = DP;
		GP(i).Vs = VS;
		GP(i).Vp = VP;
		GP(i).Density = RS;
		GP(i).Porosity = PO;
		GP(i).DifferentialPressure = PR;
		if igt == 0
			GP(i).Temperature = interp1(TParms.Depth,TParms.Temp,DP,'linear','extrap');
		end
		GP(i).WaterTableDepth = wtd(i);
	end
end
GP.Depth = Parms.Depth;
GP.Vs = Vs;
GP.Vp = Vp;
GP.Density = Density;
GP.Porosity = Porosity;
GP.DifferentialPressure = DifferentialPressure;
GP.Temperature = Temperature;
GP.WaterTableDepth = WaterTableDepth;
