function RP = RockProp(Parms)
%function RP = RockProp(Parms)
%	
%       Written by Oliver Boyd
%       V0.1 released July 5th 2017
%
%	This program calculates the geophysical parameters of a porous mineral aggregate
%	using one of 4 methods as indicated by the variable Parms.t, though t = 4 has been
%	given the most attention.
%
%       OUTPUT (SI Units)
%	RP   - structure of rock properties
%                   Parms - Input parameter structure
%           FluidPressure
%            RockPressure
%    DifferentialPressure
%                Porosity
%                       r - Mineral aggregate density
%                       K - Mineral aggregate bulk modulus
%                       G - Mineral aggregate shear modulus
%                      rs - Bulk rock density
%                      Ks - Bulk rock bulk modulus
%                      Gs - Bulk rock shear modulus
%                      Vp - P-wave velocity
%                      Vs - S-wave velocity
%                   att_p - Assummed P-wave attenuation to convert from anharmonic to seismic
%                   att_s - Assummed S-wave attenuation to convert from anharmonic to seismic
%
%       INPUT (SI Units, unless specified otherwise)
%	Parms - structure containing the following:
%	   Comp - composition structure--see MinVel
%	   p0   - zero pressure porosity
%		NOTE: Be careful that if the porosity is too high, fluid pressure will increase faster than
%		rock pressure. Recommend capping the porosity to prevent this.
%	   p0a  - porosity depth scaling in p0*exp(-p0a*dep)
%	   var  - matrix (3xn) containing the following vectors, each of length(n);
%		Depth, Pressure, Temperature (m,Pa,K)
%		If matrix is 1xn (Depth only), Pressure and Temperature are calculated 
%			in the following way:
%			Pressure = 1.01e5 + Depth*9.8*2700;
%			Temp = 300+0.022*Depth;
%		NOTE: Depth vector is the midpoint depth
%          t    - Type of relation
%               1 - Toksoz et al, 1976
%		2 - Budiansky and O'Connel
%		3 - Myung Lee (2003)
%		4 - Myung Lee (2010), currently used for NCM
%
%          The following parameters are optional. Default values are 0 if not specified.
%	   WaterTableDepth - Water table depth (m). If specifying a profile beginning at some depth where
%	       fp0 includes the effect of the water table, set wtd = 0.
%	   VadoseZoneThickness - Thickness of partially saturated region above the water table.
%	   rp0  - Rock pressure at the top of the column, should be earth surface pressure
%	       if column begins at the surface.
%	   fp0  - fluid pressure at the top of the column, should be earth surface pressure
%	       if column begins at the surface.
%	   a    - aspect ratio of inclusions, if t == 1 or 2
%
%          The following parameters are optional but, if specified, must be specified together
%	   att_p - P-wave attenuation
%	   att_s - S-wave attenuation
%	   qf    - Frequency dependence of attenuation
%
%	   The following Parms fields may be specified in order to avoid call to MinVel
%	   K       - bulk modulus
%	   G       - shear modulus
%	   Density - density
%

% For Lee's method, another useful article to cite is 
% Luo, X., Were, P., Liu, J. et al. Environ Earth Sci (2015) 73: 7019. doi:10.1007/s12665-015-4219-8
% For comparisons with Prasad's data, a composition of
%Comp(8,[1 5 66]) = [0.4 0.2 0.4]; % works best

%maxPor = 0.8;
Gravity = 9.8;
ESP = 1.01e5; % pressure at Earth's surface
minDiffP = ESP;

if length(Parms.var(:,1)) == 3
	Depth = Parms.var(1,:);
	Pressure = Parms.var(2,:);
	Temp = Parms.var(3,:);
else
	Depth = Parms.var(1,:);
	Pressure = 1.01e5 + Depth*9.8*2700;
	Temp = 300+0.022*Depth;
end
nP = length(Pressure);

% Dry
%up = 0;
%Kp = 0;
%rp = 0;

% Water as the assumed inclusion
rFluid = 1e3;
rAir = 1.225;
KFluid = 2.29e9;
KAir = 1.42e5;
up = 0;
Kp = KFluid*ones(1,nP);
rp = rFluid*ones(1,nP);
if isfield(Parms,'fp0') == 1
	fp0 = Parms.fp0;
else
	fp0 = 0;
	Parms.fp0 = fp0;
end
if isfield(Parms,'rp0') == 1
	rp0 = Parms.rp0;
	dp0 = Parms.dp0;
else
	rp0 = ESP;
	dp0 = 0;
	Parms.rp0 = rp0;
	Parms.dp0 = dp0;
end
if isfield(Parms,'WaterTableDepth') == 1
	WaterTableDepth = Parms.WaterTableDepth;
else
	WaterTableDepth = 10;
	Parms.WaterTableDepth = WaterTableDepth;
end
% Capillary fringe is part of the Vadose Zone, maybe I should call this the phreatic zone
if isfield(Parms,'CapillaryFringeThickness') == 1
	CapillaryFringeThickness = Parms.CapillaryFringeThickness;
else
	%CapillaryFringeThickness = 0;
	CapillaryFringeThickness = min(0.5,0.5*WaterTableDepth);
	Parms.CapillaryFringeThickness = CapillaryFringeThickness;
end
MaxSuction = -1000e3; % Pa
MinSaturation = 0;
if isfield(Parms,'VadoseZoneThickness') == 1
	VadoseZoneThickness = Parms.VadoseZoneThickness;
else
	%VadoseZoneThickness = min(2,WaterTableDepth);
	VadoseZoneThickness = (WaterTableDepth - CapillaryFringeThickness);
	Parms.VadoseZoneThickness = VadoseZoneThickness;
end
if isfield(Parms,'minPorSlope') == 1
	minPorSlope = Parms.minPorSlope;
else
	minPorSlope = 0.1;
	Parms.minPorSlope = minPorSlope;
end

% Considered looping this since the pressure would be dependent on the density change, 
% but the change in mineral density and moduli is extremely small.
if isfield(Parms,'K') == 0
	MV = MinVel(Parms.Comp,Pressure,Temp);
	r = MV.p;
	K = MV.K;
	u = MV.G;
else
	r = Parms.Density;
	K = Parms.K;
	u = Parms.G;
end
if isfield(Parms','maxPor')
	maxPor = Parms.maxPor;
else
	maxPor = min(r)/(rFluid + min(r));
end

% Porosity as a function of depth from Myung (see ref below)
%p = p0*exp(-p0a*var);
%disp(var)
%ggd = Gravity*gradient(Depth);
ggd = Gravity*diff([dp0 Depth]);
wden_rock = r.*(1 - 0.3);
wden_fluid = rFluid*0.3;
%FP0 = ESP + fp0;
%RP0 = ESP + rp0;
I1 = WaterTableDepth;
I2 = I1 - CapillaryFringeThickness;
I3 = I2 - VadoseZoneThickness;
wti = find(Depth <= I1);		% Above the Water Table
wti2 = find(Depth <= I1 & Depth >= I2);	% In the Capillary fringe
wti3 = find(Depth < I2 & Depth >= I3);	% In the Vadose Zone
wti4 = find(Depth < I3);		% Above the Vadose Zone
nwti = length(wti);
nwti2 = length(wti2);
nwti3 = length(wti3);
nwti4 = length(wti4);
% Sand
%a = 0.3;
%n = 3;
% Silt
%a = 0.05;
%n = 2.5;
% Clay
%a = 0.01;
%n = 1.8;
if nwti > 0 % Material above the water table
	if isfield(Parms,'alpha')
		a = Parms.alpha;
	else
		a = 0.3; % Sand from Lu et al. 2010
		Parms.alpha = a;
	end
	if isfield(Parms,'n')
		n = Parms.n;
	else
		n = 3; % Sand from Lu et al. 2010
		Parms.n = n;
	end
	WaterTablePressure = Gravity*WaterTableDepth*wden_fluid;
	%WaterTablePressure = g*(WaterTableDepth-CapillaryFringeThickness)*wden_fluid;
	FluidPressure = cumsum(ggd.*wden_fluid) - WaterTablePressure;
	%FluidPressure = fp0 + cumsum(ggd.*wden_fluid) - WaterTablePressure;
	%nrm_pCh = 1 - (wti2 - min(wti2))/(max(wti2) - min(wti2));
	% QUESTION: How does saturation change with depth in the vadose zone?
	% Related answer: John McCartney presented work on the partially saturated zone in a USGS seminar.
	%	In the presentation, he showed water retention curves and provided me a paper on the subject.
	if nwti3 > 0 % The vadose zone
		x = pi*((Depth(wti3) - I3)/(I2 - I3)) - pi/2;
		Se = (sin(x) + 1)/2;
		Se = Se.^2;
		dn = 1-Se;
		% I multiplied Sc by saturation because I couldn't match borehole velocities
		% otherwise. I think I could accomplish this by modifying the saturation profile.
		Sc = zeros(size(Se));
		j = find(Se > 0);
		Sc(j) = -(Se(j)/a).*(Se(j).^(n/(1-n)) - 1).^(1/n);
		%Sc = Sc.*Se;
		rp(wti3) = dn*(rAir - rFluid) + rFluid;
		k = find(rp < rAir); rp(k) = rAir;
		%Kp(wti3) = 1./(Se/KFluid + (1-Se)/KAir);
		Kp(wti3) = dn*(KAir - KFluid) + KFluid;
		k = find(Kp < KAir); Kp(k) = KAir;
		FluidPressure(wti3) = Sc*1e3;
	else
		dn = 0;
	end
	if nwti4 > 0 % Above the vadose zone
		rp(wti4) = rAir;
		Kp(wti4) = KAir;
		FluidPressure(wti4) = fp0;
	end
	% There are two pressures here that add to the rock pressure, the weight of the fluid and the suction.
	% According to McCartney's presentation, suction varies linearly through the vadose zone, reaching a
	% maximum at the top of the phreatic zone (?) and decreasing to zero at the top.
	% See Lu et al, 2010 for suction as a function of saturation for different deposits.
else
	dn = 0;
	FluidPressure = fp0 + cumsum(ggd.*wden_fluid);
end
RockPressure = rp0 + cumsum(ggd.*wden_rock);
% Differential pressure is referred to as effective pressure in Lu et al. 2010.
DifferentialPressure = RockPressure - FluidPressure;
k = find(DifferentialPressure < minDiffP); DifferentialPressure(k) = minDiffP;
TopVZ = WaterTableDepth - CapillaryFringeThickness - VadoseZoneThickness;
BottomVZ = WaterTableDepth - CapillaryFringeThickness;

p0aMax = log(-log(maxPor));
for j = 1:4
	EfP = DifferentialPressure/101000;
	p = exp(Parms.p0*log(EfP).^Parms.pExp + Parms.p0a);	     % (1)
	if maxPor < 1
		maxPorFunc = exp(minPorSlope*log(EfP).^Parms.pExp + p0aMax);	     % (1)
		p = (p.^10 + maxPorFunc.^10).^(1/10);
	end
	p = exp(-p);
	%maxPorFunc = exp(-exp(0.5*log(EfP).^Parms.pExp + p0aMax));	     % (1)
	%p = exp(-exp(Parms.p0*log(EfP).^Parms.pExp + Parms.p0a));	     % (1)
	%p = (1./((1./p).^10 + (1./maxPorFunc).^10)).^(1/10);
	%k = find(p < 1e-10); p(k) = 1e-10;
	%k = find(p > maxPorFunc); p(k) = maxPorFunc(k);

	wden_rock = r.*(1 - p);
	wden_fluid = rp.*p;
	%wden_fluid = rFluid*p;
	if nwti > 0
		WaterTablePressure = sum(ggd(wti).*wden_fluid(wti));
		FluidPressure = cumsum(ggd.*wden_fluid) - WaterTablePressure;
		FluidPressure(wti4) = fp0;
		if nwti3 > 0
			SC = Sc*1e3 + FluidPressure(wti3(nwti3))*(Depth(wti3) - TopVZ)/(BottomVZ - TopVZ);
			FluidPressure(wti3) = SC;
		end
	else
		FluidPressure = fp0 + cumsum(ggd.*wden_fluid);
	end
	RockPressure = rp0 + cumsum(ggd.*wden_rock);
	DifferentialPressure = RockPressure - FluidPressure;
	k = find(DifferentialPressure < minDiffP); DifferentialPressure(k) = minDiffP;
end
rs = wden_rock + rp.*p;

switch Parms.t
    case 1
	% The following obtained from Toksoz et al, Geophysics, 1976
	a = Parms.a;
	p0 = Parms.p0;
	j = find(a == 1);
	a(j) = 1 - 1e-16;
	
	% Limit minimum c/a
	minRat = K.*(3*K + 4*u)./(pi*u.*(3*K + u));
	if min(a./p0) < minRat
		RP = [];
		fprintf('Aspect ratio too low or porosity too high. The minimum ratio is %f',minRat);
		return
	end
	
	[Ka,ua] = Moduli(K,u,0,0,a,c);
	RP.Ka = Ka;
	for j = 1:1e4
		for k = 1:length(p0)
			p = p0(k)*(1 + PorosityChange(DifferentialPressure,a(k),Ka,K,u));
			c(k,:) = p;
		end
		[Ka,ua] = Moduli(K,u,0,0,a,c);
	end

	[Ks,us] = Moduli(K,u,Kp,up,a,c);
    case 2
	% Budiansky and O'Connell method
	% B. Budiansky, R.J. O'Connell
	% Elastic moduli of a cracked solid
	% Int. J. Solids Struct., 12 (1976), pp. 81–97
	e = p*3/(4*pi*a);
	v = (3*K - 2*u)./(6*K + 2*u);
	vs = v.*(1 - 16*e/9);
	Ks = K.*(1 - 16*(1 - vs.^2)*e./(9*(1 - 2*vs.^2)));
	us = u.*(1 - 32*(1 - vs).*(5 - vs).*e./(45*(2 - vs)));
    case 3
	% Myung Lee, USGS Survey Bulletin 2214
	% I think the following equation is only appropriate for porosity being a linear function
	% of effective pressure where dphi_dp is constant. For a more natural log-log scaling of
	% porosity with effective pressure, perhaps m is a constant.
	dphi_dp = gradient(p)./gradient(DifferentialPressure*1e-6);
	if isfield(Parms,'m1')
		m = 1 + Parms.m1*exp(Parms.m2*dphi_dp);
	else
		m = 1 + 4.95289*exp(5212*dphi_dp);
	end
	B = 1 - (1 - p).^3.8;
	M = 1./((B - p)./K + p./Kp);
	Cv = sum(Parms.Comp(55:70)); % Clay content
	G = 0.9552 + 0.0448*exp(-Cv/0.06714);
	% Original equation by Lee (2003).
	if isfield(Parms,'n1') == 1
		n = (10.^(Parms.n1 - Parms.n2*log10(DifferentialPressure*1e-6)))./m;
	else
		n = (10.^(0.426 - 0.235*log10(DifferentialPressure*1e-6)))./m;
	end
	Ks = K.*(1 - B) + M.*B.^2;
	us = (u.*K.*(1 - B)*G^2.*(1 - p).^(2*n) + u.*B.^2.*M*G^2.*(1 - p).^(2*n))./ ...
		(K + 4*u.*(1 - G^2*(1 - p).^(2*n))/3);
	RP.B = B;
	RP.n = n;
	RP.m = m;
	RP.Vp = sqrt((Ks + 4*us/3)./rs);
	RP.Vs = sqrt(us./rs);
    case 4
	if isfield(Parms,'A1') == 1
		A1 = Parms.A1;
		A2 = Parms.A2;
		A2exp = Parms.A2exp;
	else
		% Original approximate coeffiecients from Lee when porosity in Biot coeffients are 
		% not normalized by maxPor
		%A1 = 35;
		%A2 = 74;
		%A2exp = -0.3682;
		% When using p/maxPor in Biot coefficients, using grid search to match P- and S-velocity curves 
		% and porosity from Zimmer (2003) data Gulf of Mexico saturated sands
		% olboyd/Documents/Science/Data/Seismic/National/NCM/Calibration/ZimmerMatchPorosity.m
		% p0 = 0.066, p0a = -0.178, and pExp = 0.5;
		%A1 = 11.025288436095856;
		%A2 = 30.825849167011931;
		A2exp = -0.4;
		%M2 = 9.723002325376470;
		% Coefficients for matching P-vel are A1 = 25.15 and A2 = 70.50 (incr of 0.05);
		%A1 = 25.05; 
		%A2 = 71.20;
		A1 = 11.025288436095856;
		A2 = 30.825849167011931;
		%A2exp = -0.3;
		A1exp = 0;
%A1 = 50;
%A2 = 70;
		Parms.A1 = A1;
		Parms.A2 = A2;
		Parms.A2exp = A2exp;
%A1exp = -0.1;
	end
	if isfield(Parms,'M1') == 1
		M1 = Parms.M1;
		M2 = Parms.M2;
		M2exp = Parms.M2exp;
	else
		M1 = 2;
		%M2 = 6;
		M2 = 9.723002325376470;
		M2exp = -0.6;
%M2exp = -0.7;
		Parms.M1 = M1;
		Parms.M2 = M2;
		Parms.M2exp = M2exp;
	end
	%TransExp = 3;
	TransExp = 10;
	DP = DifferentialPressure*1e-6; % Differential Pressure in MPa

	% a1 original function to match Lee (2010)
	%a1 = 100*DP.^-0.3;
	%A = 1./(((1./a0).^TransExp + (1./a1).^TransExp).^(1/TransExp));

	a1 = A1*DP.^A1exp;
	a2 = A2*DP.^A2exp;
	%a2 = A2*DP.^-0.3682;
	%a2 = A2*DP.^-0.3;
	%a2 = A2*DP.^-0.5;
	% QUESTION: What is the correct value of a3?
	a3 = 2;
	a12 = 1./(((1./a1).^TransExp + (1./a2).^TransExp).^(1/TransExp));
	%A = (a12.^TransExp + a3.^TransExp).^(1/TransExp);
	A = a12;

	%m2p = 6*(DifferentialPressure*1e-6).^-0.6;
	%Mp = (m2p.^10 + 2.^10).^(1/10);
	%gp = (1 + Mp.*A)./(1 + A);
	%Bp = p.*(1 + gp.*A)./(1 + gp.*A.*p);
	%Bp = p.*(1 + A)./(1 + A.*p);
	Bp = (p/maxPor).*(1 + A)./(1 + A.*(p/maxPor));

	% m2 and m3 original functions from Lee (2010)
	m1 = A1 - 0*DP;
	m2 = M2*(DP).^M2exp;
	%m2 = M2*(DP).^-0.6;
	m3 = M1 - 0.0*DP;
	%m2 = 6*(DP).^-0.6;
	%m3 = 2 - 0.0*DP;

	% Following values used to match average boreholes, vp and vs.
	%m1 = 18 - 0*DP;
	%m2 = 3.7*(DP).^-0.4;
	%m2 = 2.7*(DP).^-0.5;
	%m2 = 6*(DP).^-2.7;
	%m3 = 1 - 0.0*DP;

	%m3 = 0;
	%m12 = 1./(((1./m1).^10 + (1./m2).^10).^(1/10));
	%M = (m12.^10 + m3.^10).^(1/10);
	%m12 = 1./(((1./m1).^TransExp + (1./m2).^TransExp).^(1/TransExp));
	m12 = m2;
	M = (m12.^TransExp + m3.^TransExp).^(1/TransExp);
	g = (1 + M.*A)./(1 + A);
	%if length(wti) > 0
		%g(wti) = g(wti(nwti));
	%end

	% I know that Bp must approach Bs as saturation approaches 0. I also know that a sudden change in g
	% produces a discontinuity in velocity that is not observed. What if g approaches 1 as saturation 
	% approaches 0. This would cause Bs to approach Bp as saturation approaches 0. Unfortunately, this
	% still causes a large increase in shear wave velocity that is not observed.
	%g(wti4) = 1;
	%g(wti3) = g(wti3) - dn.*(g(wti3) - 1);

	% Lee contends that above the water table, g = 1
	% This assumption produces a large jump (> 200 m/s) in velocity at the water table
	% that is not seen in the velocity profiles.
	%g(wti) = 1;

	%Bs = p.*(1 + g.*A)./(1 + g.*A.*p);
	Bs = (p/maxPor).*(1 + g.*A)./(1 + g.*A.*(p/maxPor));

	% QUESTION: How does Bp change above the Water Table?
	% Let's consider for Bp whether or not the pore space material
	% offers compressible resistance. In other words, can it flow easily at
	% seismic frequencies. If yes (a gas), Bp equals Bs. If not (a fluid)
	% Bp equals the usual formulation.
	Bp(wti4) = Bs(wti4); % I had this originally but don't know where I got it.
	Bp(wti3) = Bp(wti3) - dn.*(Bp(wti3) - Bs(wti3));

	Kav = 1./((Bp - (p/maxPor))./K + (p/maxPor)./Kp);
	Ks = K.*(1 - Bp) + Kav.*Bp.^2;
	us = u.*(1 - Bs);
	
	% assume this method was calibrated on high frequency data.
	% We therefore need to adjust for data measured at seismc frequencies
	% QUESTION: What attenuation functions should be used for p- and s-waves?
	Vp = sqrt((Ks + 4*us/3)./rs);
	Vs = sqrt(us./rs);
	if Parms.Attenuate == 1
		if isfield(Parms,'AagaardModel') == 1
			% Aagaard et al. functionality from the bay area
			Qs = -16 + 104.13*Vs/1e3 - 25.225*(Vs/1e3).^2 + 8.2184*(Vs/1e3).^3;
			Qs = max(5,Qs);
			Qp = 2*Qs;
			att_p = 1./Qp;
			att_s = 1./Qs;
			qf = 0;

			%att_s = 0.01*(DifferentialPressure/1e6 + 0.01).^-0.6;
			%ap0 = 1*(DifferentialPressure/1e6 + 2).^-1.5;
			%ap1 = (4/9)*att_s;
			%att_p = (ap0.^2 + ap1.^2).^(1/2);
			%qf = 0.25;

			Parms.att_p = att_p;
			Parms.att_s = att_s;
			Parms.qf = qf;
		else
			att_p = Parms.att_p;
			att_s = Parms.att_s;
			qf = Parms.qf;
		end
		Vp = Vp.*(1 - att_p*0.5*cos(pi*qf/2));
		Vs = Vs.*(1 - att_s*0.5*cos(pi*qf/2));
	end
end

RP.Parms = Parms;
RP.Bp = Bp;
RP.Bs = Bs;
RP.M = M;
RP.A = A;
RP.g = g;
RP.maxPor = maxPor;
if exist('Se','var')
	RP.Se = Se;
end
RP.wti3 = wti3;
RP.FluidPressure = FluidPressure;
RP.dn = dn;
RP.dwti3 = Depth(wti3);
RP.rp = rp;
RP.kp = Kp;
RP.RockPressure = RockPressure;
RP.DifferentialPressure = DifferentialPressure;
RP.Porosity = p;
RP.r = r; % Solid density
RP.K = K; % Solid bulk modulus
RP.G = u; % Solid shear modulus
RP.rs = rs; % Bulk density
RP.Ks = Ks; % Bulk bulk modulus
RP.Gs = us; % Bulk shear modulus
RP.Vp = Vp; % P-wave velocity
RP.Vs = Vs; % S-wave velocity

function dpOp = PorosityChange(P,a,Ka,K,u)
oma2 = 1 - a^2;
tKp4u = 3*K + 4*u;

l = 2*pi*a*(acos(a) - a*sqrt(oma2))/oma2^(3/2);

E1 = 6*u*l/(2*pi*tKp4u);
E2 = 6*u*(3*l - 4*pi)/(4*pi*tKp4u);
E3 = a^2*(3 - 9*l/(4*pi))*(6*K + 2*u)/(2*oma2*tKp4u) + ...
	6*u*l/(8*pi*tKp4u);
% Toksoz et al. 1976
E4 = ((3 - 9*l/(4*pi))*(6*K + 2*u)/(2*oma2*tKp4u) - ...
	3*u*(1 - l/pi)/tKp4u)/2;
% Cheng and Toksoz, 1979
%E4 = ((3 - 9*l/(4*pi))*(6*K + 2*u)/(2*oma2*tKp4u) - ...
	%3*(1 - l/pi)/tKp4u)/2;

dpOp = -(P./Ka)/(E1 - E2*E3/(E3 + E4));

function [Ks,us] = Moduli(K,u,Kp,up,a,c)

oma2 = 1 - a.^2;

A = up/u - 1;
B = (Kp/K - up/u)/3;
R = 3*u/(3*K + 4*u);
p = a.*(acos(a) - a.*sqrt(oma2))./oma2.^(3/2);
g = a.^2.*(3*p - 2)./oma2;

tm4R = 3 - 4*R;

F1 = 1 + A*(3*(g + p)/2 - R*(3*g/2 + 5*p/2 - 4/3));
F2 = 1 + A*(1 + 3*(g + p)/2 - R*(3*g + 5*p)/2) + B*tm4R ...
	+ A*(A + 3*B)*tm4R*(g + p - R*(g - p + 2*p.^2))/2;
%F3 = 1 + A*(R*(2 - p) + (1 + a^2)*(3*p - 2)*(R - 1)/oma2)/2
% Toksoz et al, 1976
F3 = 1 + A*(R*(2 - p) + (1 + a.^2).*g*(R - 1)./a.^2)/2;
% Cheng and Toksoz, 1979
%F3 = 1 + A*(R*(2 - p) + (1 + a^2)*g*(R - 1)/(2*a))/2;
F4 = 1 + A*(3*p + g - R*(g - p))/4;
F5 = A*(R*(g + p - 4/3) - g) + B*p*tm4R;
F6 = 1 + A*(1 + g - R*(g + p)) + B*(1 - p)*tm4R;
F7 = 2 + A*(9*p + 3*g - R*(5*p + 3*g))/4 + B*p*tm4R;
F8 = A*(1 - 2*R + g*(R - 1)/2 + p*(5*R - 3)/2) + B*(1 - p)*tm4R;
F9 = A*(g*(R - 1) - R*p) + B*p*tm4R;

nP = length(c(1,:));
Tiijj = 3*F1./F2;
Tijij = 2./F3 + 1./F4 + (F4.*F5 + F6.*F7 - F8.*F9)./(F2.*F4) + Tiijj/3;

Tiijj = Tiijj'*ones(1,nP);
Tijij = Tijij'*ones(1,nP);

C = (Kp - K)*sum(c.*Tiijj,1)/(9*K + 12*u);
Ks = (K + 4*C*u)./(1 - 3*C);

%whos
%pause
C1 = (up - u)*sum(c.*(Tijij - Tiijj/3),1)/(25*u*(3*K + 4*u));
C2 = K + 2*u;
C3 = u*(9*K + 8*u);
us = (C1*C3 + u)./(1 - 6*C2*C1);
%pause
